"use strict";
function Cleanable(options) {
    if (options === void 0) { options = { id: '' }; }
    return function (target, key) {
        var metadataId = "__cleanable__subscriptions__" + options.id;
        var baseMetadata = Reflect.getOwnMetadata(metadataId, target) || [];
        Reflect.defineMetadata(metadataId, baseMetadata.concat([key]), target);
    };
}
exports.Cleanable = Cleanable;
function Clean(options) {
    if (options === void 0) { options = { id: '', before: true }; }
    options.id = options.id || '';
    if (typeof options.before !== 'boolean') {
        options.before = true;
    }
    return function (target, key, descriptor) {
        var method = descriptor.value;
        descriptor.value = function () {
            var _this = this;
            if (!options.before) {
                method.apply(this, arguments);
            }
            var metadata = Reflect.getOwnMetadata("__cleanable__subscriptions__" + options.id, target);
            if (metadata) {
                metadata.forEach(function (property) {
                    if (property && _this[property]) {
                        if (_this[property].unsubscribe) {
                            _this[property].unsubscribe();
                        }
                        else if (Array.isArray(_this[property])) {
                            // Handle array of subscriptions
                            _this[property].forEach(function (x) {
                                if (x.unsubscribe) {
                                    x.unsubscribe();
                                }
                            });
                        }
                    }
                });
            }
            if (options.before) {
                return method.apply(this, arguments);
            }
        };
        return descriptor;
    };
}
exports.Clean = Clean;
